@@ -9,7 +9,7 @@
 package Dancer::Plugin::Swagger;
 our $AUTHORITY = 'cpan:YANICK';
 # ABSTRACT: create Swagger documentation of the app REST interface 
-$Dancer::Plugin::Swagger::VERSION = '0.1.0';
+$Dancer::Plugin::Swagger::VERSION = '0.2.0';
 use strict;
 use warnings;
 
@@ -28,7 +28,6 @@ use Class::Load qw/ load_class /;
 
 use Path::Tiny;
 use File::ShareDir::Tarball;
-use Class::Load qw/ load_class /;
 
 sub import {
     $Dancer::Plugin::Swagger::FIRST_LOADED ||= caller;
@@ -129,16 +128,14 @@ my $plugin = __PACKAGE__->instance;
 if ( $plugin->show_ui ) {
     my $base_url = $plugin->ui_url;
 
-    get $base_url => sub {
+    get $base_url => sub { redirect $base_url . '/?url=/swagger.json' };
+
+    get $base_url . '/' => sub {
         my $file = $plugin->ui_dir->child('index.html');
 
         send_error "file not found", 404 unless -f $file;
 
-        my $content = $file->slurp;
-        $content =~ s/UI_DIR/$base_url/g;
-        $content =~ s!SWAGGER_URL!uri_for( '/swagger.json'  )!eg;
-
-        $content;
+        return $file->slurp;
     };
 
     get $base_url.'/**' => sub {
@@ -191,8 +188,46 @@ register swagger_path => sub {
     # we don't process HEAD
     @routes = grep { $_->method ne 'head' } @routes;
 
+    my $description;
+    if( @_ and not ref $_[0] ) {
+        $description = shift;
+        $description =~ s/^\s*\n//;
+        
+        $description =~ s/^$1//mg
+            if $description =~ /^(\s+)/;
+    }
+
     my $arg = shift @_ || {}; 
 
+    $arg->{description} = $description if $description;
+
+    # groom the parameters
+    if ( my $p = $arg->{parameters} ) {
+        if( ref $p eq 'HASH' ) {
+            $_ = { description => $_ } for grep { ! ref } values %$p;
+            $p = [ map { +{ name => $_, %{$p->{$_}} } } sort keys %$p ];
+        }
+
+        # deal with named parameters
+        my @p;
+        while( my $k = shift @$p ) {
+            unless( ref $k ) { 
+                my $value = shift @$p;
+                $value = { description => $value } unless ref $value;
+                $value->{name} = $k;
+                $k = $value;
+            }
+            push @p, $k;
+        }
+        $p = \@p;
+
+        # set defaults
+        $p = [ map { +{ in => 'query', type => 'string', %$_ } } @$p ];
+        
+        $arg->{parameters} = $p;
+    }
+
+
     for my $route ( @routes ) {
         my $path = Dancer::Plugin::Swagger::Path->new(%$arg, route => $route);
 
@@ -234,13 +269,14 @@ sub swagger_response {
 register swagger_response => \&swagger_response;
 
 register swagger_definition => sub {
-    my %defs = @_;
+    my( $name, $def ) = @_;
 
     $plugin->doc->{definitions} ||= {};
 
-    while( my($k,$v)=each%defs ) {
-        $plugin->doc->{definitions}{$k} = $v;
-    }
+    $plugin->doc->{definitions}{$name} = $def;
+
+    return { '$ref', => '#/definitions/'.$name };
+
 };
 
 register_plugin;
@@ -259,7 +295,7 @@ Dancer::Plugin::Swagger - create Swagger documentation of the app REST interface
 
 =head1 VERSION
 
-version 0.1.0
+version 0.2.0
 
 =head1 SYNOPSIS
 
@@ -352,7 +388,7 @@ Defaults to C<false>.
 
 =head1 PLUGIN KEYWORDS
 
-=head2 swagger_path \%args, $route
+=head2 swagger_path $description, \%args, $route
 
     swagger_path {
         description => 'Returns info about a judge',
@@ -363,6 +399,31 @@ Defaults to C<false>.
 
 Registers a route as a swagger path item in the swagger document.
 
+C<%args> is optional.
+
+The C<$description> is optional as well, and can also be defined as part of the 
+C<%args>.
+
+    # equivalent to the main example
+    swagger_path 'Returns info about a judge',
+    get '/judge/:judge_name' => sub {
+        ...;
+    };
+
+If the C<$description> spans many lines, it will be left-trimmed.
+
+    swagger_path q{ 
+        Returns info about a judge.
+
+        Some more documentation can go here.
+
+            And this will be seen as a performatted block
+            by swagger.
+    }, 
+    get '/judge/:judge_name' => sub {
+        ...;
+    };
+
 =head3 Supported arguments
 
 =over
@@ -383,9 +444,13 @@ Defaults to the route's path.
 
 The path item's description.
 
+=item tags
+
+Optional arrayref of tags assigned to the path.
+
 =item parameters
 
-List of parameters for the path item. Must be an arrayref.
+List of parameters for the path item. Must be an arrayref or a hashref.
 
 Route parameters are automatically populated. E.g., 
 
@@ -401,6 +466,44 @@ is equivalent to
     },
     get '/judge/:judge_name' => { ... };
 
+If the parameters are passed as a hashref, the keys are the names of the parameters, and they will
+appear in the swagger document following their alphabetical order.
+
+If the parameters are passed as an arrayref, they will appear in the document in the order
+in which they are passed. Additionally, each parameter can be given as a hashref, or can be a 
+C<< name => arguments >> pair. 
+
+In both format, for the key/value pairs, a string value is considered to be the 
+C<description> of the parameter.
+
+Finally, if not specified explicitly, the C<in> argument of a parameter defaults to C<query>,
+and its type to C<string>.
+
+    parameters => [
+        { name => 'bar', in => 'path', required => 1, type => 'string' },
+        { name => 'foo', in => 'query', type => 'string', description => 'yadah' },
+    ],
+
+    # equivalent arrayref with mixed pairs/non-pairs
+
+    parameters => [
+        { name => 'bar', in => 'path', required => 1, type => 'string' },
+        foo => { in => 'query', type => 'string', description => 'yadah' },
+    ],
+
+    # equivalent hashref format 
+    
+    parameters => {
+        bar => { in => 'path', required => 1, type => 'string' },
+        foo => { in => 'query', type => 'string', description => 'yadah' },
+    },
+
+    # equivalent, using defaults
+    parameters => {
+        bar => { in => 'path', required => 1 },
+        foo => 'yadah',
+    },
+
 =item responses
 
 Possible responses from the path. Must be a hashref.
@@ -502,6 +605,20 @@ Adds a schema (or more) to the definition section of the Swagger document.
         }
     };
 
+The function returns the reference to the definition that can be then used where
+schemas are used.
+
+    my $Judge = swagger_definition 'Judge' => { ... };
+    # $Judge is now the hashref '{ '$ref' => '#/definitions/Judge' }'
+    
+    # later on...
+    swagger_path {
+        responses => {
+            default => { schema => $Judge },
+        },
+    },
+    get '/judge/:name' => sub { ... };
+
 =head1 EXAMPLES
 
 See the F<examples/> directory of the distribution for a working example.
